import matplotlib
matplotlib.use("GTKAgg", warn=False, force=True)
import nest
import numpy as np
import pylab
import time


# simulation parameters
T = 1000.                  # simulation time (ms)
dt = 0.1                   # simulation resolution (ms)

# network parameters
N_L5E = 4850               # number of neurons in L5E
N_L5I = 1065               # number of neurons in L5I
      
Nsyn_L5E_L5E = 2038173     # number of synapses with presynptic neuron (pre) in L5E and postsynaptic neuron (post) in L5E 
Nsyn_L5E_L5I = 319602      # number of synapses with pre in L5E and post in L5I
Nsyn_L5I_L5I = 430775      # number of synapses with pre in L5I and post in L5I
Nsyn_L5I_L5E = 2411184     # number of synapses with pre in L5I and post in L5E

K_L5E_ext = 2000           # indegree of excitatory neurons to external poisson drive
K_L5I_ext = 1900           # indegree of inhibitory neurons to external poisson drive

# neuron parameters
neuron_params = {
      'C_m'       : 250.,    # pF
      'I_e'       : 0.0,     # nA
      'tau_m'     : 10.0,    # ms
      't_ref'     : 2.0,     # ms
      'tau_syn_ex': 0.5,     # ms
      'tau_syn_in': 0.5,     # ms
      'V_reset'   : -65.0,   # mV
      'E_L'       : -65.0,   # mV
      'V_th'      : -50.0    # mV
}

# synapse parameters
w = 87.8                   # mean excitatory weight (pA)
sigma_w = 8.8              # standard deviation of excitatory weight (pA)
g = -4.                    # relative inhibitory weight 

de = 1.5                   # mean spike transmission delay for excitatory presynaptic neurons (ms)
sigma_de = 0.75            # standard deviation 
di = 0.8                   # mean spike transmission delay for inhibitory presynaptic neurons (ms)
sigma_di = 0.4             # standard deviation
      
# external input parameters
bg_rate = 8.               # external poisson rate
perturbation = True        # external input toggler



# simulation setup
nest.ResetKernel()
nest.SetKernelStatus({'resolution': dt})      # set simulation resolution"

# populations creation
pop_L5E = nest.Create('iaf_psc_exp', N_L5E, neuron_params)
pop_L5I = nest.Create('iaf_psc_exp', N_L5I, neuron_params)


# excitatory connections (connections from L5E)

# specifying the synapse parameters
weight_dict = {
      'distribution': 'normal_clipped',
      'mu': w,
      'sigma': sigma_w,
      'low': 0.0
}
delay_dict = {
      'distribution': 'normal_clipped',
      'mu': de,
      'sigma': sigma_de,
      'low' : 0.1
}
syn_dict = {
      'weight': weight_dict,
      'delay': delay_dict
}

# connections to L5E
conn_dict = {'rule': 'fixed_total_number', 'N': Nsyn_L5E_L5E}
nest.Connect(pop_L5E, pop_L5E, conn_dict, syn_dict)

# connections to L5I
conn_dict = {'rule': 'fixed_total_number', 'N': Nsyn_L5E_L5I}
nest.Connect(pop_L5E, pop_L5I, conn_dict, syn_dict)


# inhibitory connections (connections from L5I)

# specifying the synapse parameters
weight_dict = {
      'distribution': 'normal_clipped',
      'mu': w*g,
      'sigma': np.abs(sigma_w*g),
      'high': 0.0
}
delay_dict = {
      'distribution': 'normal_clipped',
      'mu': di,
      'sigma': sigma_di,
      'low' : 0.1
}
syn_dict = {
      'weight': weight_dict,
      'delay': delay_dict
}

# connections to L5E
conn_dict = {'rule': 'fixed_total_number', 'N': Nsyn_L5I_L5E}
nest.Connect(pop_L5I, pop_L5E, conn_dict, syn_dict)

# connections to L5I
conn_dict = {'rule': 'fixed_total_number', 'N': Nsyn_L5I_L5I}
nest.Connect(pop_L5I, pop_L5I, conn_dict, syn_dict)


# other layers

# connections to L5E
poisson_generator_L5E = nest.Create('poisson_generator', params = {'rate': bg_rate*K_L5E_ext})
nest.Connect(poisson_generator_L5E, pop_L5E, 'all_to_all', {'weight': w, 'delay': de})

# connections to L5I
poisson_generator_L5I = nest.Create('poisson_generator', params = {'rate': bg_rate*K_L5I_ext})
nest.Connect(poisson_generator_L5I, pop_L5I, 'all_to_all', {'weight': w, 'delay': de})


# spike detectors
sd_L5E = nest.Create('spike_detector')
nest.Connect(pop_L5E, sd_L5E, 'all_to_all')

sd_L5I = nest.Create('spike_detector')
nest.Connect(pop_L5I, sd_L5I, 'all_to_all')


# external input
if perturbation:
      
      # parameters
      N_pert = 1000
      Nsyn_L5E_pert = 500000
      start = 100.
      stop = 300.
      pert_rate = 8.
      
      poisson_generator_stimulus = nest.Create('poisson_generator', params={'rate': pert_rate, 'start': start, 'stop':stop})
      parrot_neurons = nest.Create('parrot_neuron', N_pert)
      nest.Connect(poisson_generator_stimulus, parrot_neurons, 'all_to_all')
      
      weight_dict = {
      'distribution': 'normal_clipped',
      'mu': w,
      'sigma': sigma_w,
      'low': 0.0
      }
      delay_dict = {
      'distribution': 'normal_clipped',
      'mu': de,
      'sigma': sigma_de,
      'low' : 0.1
      }
      syn_dict = {'weight': weight_dict, 'delay': delay_dict}
      
      conn_dict = {'rule': 'fixed_total_number', 'N': Nsyn_L5E_pert}
      nest.Connect(parrot_neurons, pop_L5E, conn_dict, syn_dict) 


#simulation 
start_time = time.clock()
nest.Simulate(T)
print "Elapsed time: %fs" % (time.clock() - start_time)

# collecting results
spike_senders_L5E = nest.GetStatus(sd_L5E)[0]['events']['senders']
spike_times_L5E = nest.GetStatus(sd_L5E)[0]['events']['times']
spike_senders_L5I = nest.GetStatus(sd_L5I)[0]['events']['senders']
spike_times_L5I = nest.GetStatus(sd_L5I)[0]['events']['times']

# stack the data from the two populations
spike_senders = np.hstack((spike_senders_L5E, spike_senders_L5I))
spike_times = np.hstack((spike_times_L5E, spike_times_L5I))
pylab.plot(spike_times, spike_senders, 'b.', markersize=2)
pylab.xlim(0,T)
pylab.ylim(pop_L5E[0], pop_L5I[-1])
pylab.xlabel('time (ms)')
pylab.ylabel('neuron id')
pylab.show()

rate_E = nest.GetStatus(sd_L5E, 'n_events')[0]/(T/1000.0) / N_L5E
print "Firing rate E = %.1f spikes/s" % (rate_E)
rate_I = nest.GetStatus(sd_L5I, 'n_events')[0]/(T/1000.0) / N_L5I
print "Firing rate I = %.1f spikes/s" % (rate_I)

# nest.raster_plot.from_device(sd_L5E, hist=True)
# nest.raster_plot.from_device(sd_L5I, hist=True)
# nest.raster_plot.show()
