import matplotlib
matplotlib.use("GTKAgg", warn=False, force=True)
import pylab
import nest
from nest import raster_plot
import time

from plot_weight_matrices import plot_weight_matrices

# nest setup
nest.ResetKernel()

# +-------------------------------------------------------------------+
# | General Parameters                                                |
# +-------------------------------------------------------------------+

# Population parameters
model = 'iaf_psc_exp'

cell_params = {'C_m': 0.25,
               'I_e': 0.0,
               'tau_m': 20.0,
               't_ref': 2.0,
               'tau_syn_ex': 5.0,
               'tau_syn_in': 5.0,
               'V_reset': -70.0,
               'E_L': -65.0,
               'V_th': -50.0,
               'tau_minus': 20.0
               }


# Other simulation parameters
e_rate = 80.
in_rate = 300.

n_stim_test = 5
n_stim_pairing = 20
dur_stim = 15

pop_size = 500

ISI = 90.
start_test_pre_pairing = 200.
start_pairing = 1500.
start_test_post_pairing = 700.

simtime = (start_pairing + start_test_post_pairing
           + ISI * (n_stim_pairing + n_stim_test) + 550.)

# Initialisations of the different types of populations
IAddPre = []
IAddPost = []

# +-------------------------------------------------------------------+
# | Creation of neuron populations                                    |
# +-------------------------------------------------------------------+

# Neuron populations
pre_pop = nest.Create(model, pop_size, cell_params)
post_pop = nest.Create(model, pop_size, cell_params)

# Test of the effect of activity of the pre_pop population on the post_pop
# population prior to the "pairing" protocol : only pre_pop is stimulated
for i in range(n_stim_test):
    IAddPre.append(nest.Create('poisson_generator',
                               pop_size,
                               {'rate': in_rate,
                                'start': start_test_pre_pairing + ISI * (i),
                                'stop': start_test_pre_pairing + ISI * (i)+dur_stim
                               }))

# Pairing protocol : pre_pop and post_pop are stimulated with a 10 ms
# difference
for i in range(n_stim_pairing):
    IAddPre.append(nest.Create('poisson_generator',
                               pop_size,
                               {'rate': in_rate,
                                'start': start_pairing + ISI * (i),
                                'stop': start_pairing + ISI * (i) + dur_stim
                               }))
    IAddPost.append(nest.Create('poisson_generator',
                                pop_size,
                                {'rate': in_rate,
                                 'start': start_pairing + ISI * (i) + 10.,
                                 'stop': start_pairing + ISI * (i) + 10.+dur_stim
                                }))

# Test post pairing : only pre_pop is stimulated (and should trigger activity
# in Post)
for i in range(n_stim_test):
    IAddPre.append(nest.Create('poisson_generator',
                               pop_size,
                               {'rate': in_rate,
                                'start': (start_pairing
                                          + ISI * (n_stim_pairing)
                                          + start_test_post_pairing
                                          + ISI * (i)),
                                'stop':  (start_pairing
                                          + ISI * (n_stim_pairing)
                                          + start_test_post_pairing
                                          + ISI * (i))
                                          + dur_stim
                                }))

# Noise inputs
INoisePre = nest.Create('poisson_generator',
                        pop_size,
                        {'rate': e_rate, 'start': 0., 'stop': simtime})
INoisePost = nest.Create('poisson_generator',
                         pop_size,
                         {'rate': e_rate, 'start': 0., 'stop': simtime})

# +-------------------------------------------------------------------+
# | Creation of connections                                           |
# +-------------------------------------------------------------------+

# Connection parameters
JEE = 3.

# Connection type between noise poisson generator and excitatory populations
ee_connector = {'rule': 'one_to_one'}
syn_dict = {
      'weight': JEE * 0.05,
      'delay': 1.0
}


# Noise projections
nest.Connect(INoisePre, pre_pop, ee_connector, syn_dict)
nest.Connect(INoisePost, post_pop, ee_connector, syn_dict)

# Additional Inputs projections
for i in range(len(IAddPre)):
    nest.Connect(IAddPre[i], pre_pop, ee_connector, syn_dict)
for i in range(len(IAddPost)):
    nest.Connect(IAddPost[i], post_pop, ee_connector, syn_dict)

# Plastic Connections between pre_pop and post_pop
stdp_model = {'model': 'stdp_synapse',
              'tau_plus': 20.,
              'lambda': 0.02,
              'alpha': 0.02,
              'Wmax': 0.9,
              'weight': 0.0
             }

fp_connector = {'rule': 'fixed_total_number', 'N': int(0.5*pop_size*pop_size)}
nest.Connect(pre_pop, post_pop, fp_connector, stdp_model)

# plot_weight_matrices(pre_pop, post_pop)

# +-------------------------------------------------------------------+
# | Simulation and results                                            |
# +-------------------------------------------------------------------+

# Record neurons' potentials
# mult = nest.Create('multimeter')
# nest.Connect(mult, pre_pop)
# nest.Connect(mult, post_pop)

# Record spikes
sprec_pre = nest.Create('spike_detector')
sprec_post = nest.Create('spike_detector')
nest.Connect(pre_pop, sprec_pre)
nest.Connect(post_pop, sprec_post)

# Run simulation
start_time = time.clock()
nest.Simulate(simtime)
print "Elapsed time: %fs" % (time.clock() - start_time)

raster_plot.from_device(sprec_pre)
raster_plot.from_device(sprec_post)
raster_plot.show()

# plot_weight_matrices(pre_pop, post_pop)